# Lineární zobrazení

- $U = R^4$ - před zobrazením
- $V = R^3$ - po zobrazení
- $\mathbb{L} : U \to V$

Nazývá se také **homomorfizmus**.

### Ověření linearity zobrazení

- zkontrolovat, že platí
	- $\mathbb{L}(V + V) = \mathbb{L}(V) + \mathbb{L}(V)$
	- $\mathbb{L}(k \cdot V) = k \cdot \mathbb{L}(V)$

### Jádro

- všechny LK vektorů před zobrazením, které se po zobrazení rovnají 0
- zjištění přes zjištění LK
	- $Ker \space \mathbb{L} = \{ \mathbb{L}(V) = 0 \}$
- zápis: $Ker \space \mathbb{L} = \langle \vec{u}; \vec{v} \rangle$

Zapíšu zobrazení do matice, po provedení GJEM zjistím, které prvky se zobrazí na nulový vektor (tedy si vyjádřím např. $a, b, c$).

### Obraz

- všechny LK vektorů po zobrazení
- zápis: $Im \space \mathbb{L} = \langle \vec{u}; \vec{v} \rangle$

Vypočítám jej opět zapsáním zobrazení do matice a provedením GJEM. Obrazem je poté LN množina vektorů (podobné jako u báze).

### Identické zobrazení

Zobrazení definované vztahem $F(x) = (x)$.

### Prosté zobrazení

Každý prvek z prostoru $U$ se zobrazí přesně na jeden prvek z prostoru $V$.
- $\dim(Ker \space \mathbb{L}) = \{\vec{o}_{U}\}$

### Izomorfní zobrazení

Lineární zobrazení je **izomorfizmem**, pokud je **prosté** a zároveň $\dim(Im \space \mathbb{L}) = \dim(V)$.

## Matice lineárního zobrazení

Nejsnadnější způsob, jak počítačově popsat lineární zobrazení.

Znázorňuje vztah souřadnicemi prvku vzhledem k jedné bázi a souřadnicemi zobrazení prvku vzhledem k druhé bázi.
- **Dimenze obrazu** lineárního zobrazení $\mathbb{L}$ je **stejná jako hodnost matice** lineárního zobrazení.
- Pokud je matice lineárního zobrazení **regulání**, lineární zobrazení je **izomorfizmus**.

**Postup**:
- Určete matici zobrazení $\mathbb{L}$ v bázích $B_{1}$ a $B_{2}$.
1. Vektory první báze **zobrazím pomocí lineárního zobrazení**.
2. Zobrazené vektory napíšu do sloupců matice $A_{2}$.
3. Do matice $A_{1}$ napíšu do sloupců vektory ze druhé báze.
4. Matice **spojím** do matice $A = [A_{1} \mid A_{2}]$, kterou vyřeším pomocí GJEM.
5. Na **levé straně** díky GJEM dostanu **jednotkovou matici** a na **pravé straně** vznikne **matice lineárního zobrazení**.

### Matice přechodu

Matice identického lineárního zobrazení vzhledem k bázím $B_{1}$ a $B_{2}$.

Postup je stejný jako u matice lineárního zobrazení, jen prvky první báze nezobrazuji a rovnou je zapíšu do matice.

### Složené zobrazení

Nechť $\mathbb{L}_{1} : U \to V, \mathbb{L}_{2} : V \to W$ a báze v $U, V, W$ jsou $C, D, E$. A je matice $\mathbb L_1$ vzhledem k bázím $C, D$ a $B$ je matice $\mathbb L_{2}$ vhledem k bázím $D, E$.

Složené zobrazení $\mathbb L = \mathbb L_{2} \circ \mathbb L_{1} : U \to W$ je lineární a jeho matice vzhledem k bázím $C, E$ je rovna matici $B \cdot A$.

Důsledky:
- Pro uvedené matice lin. zobr. platí: $hod(B \cdot A) \leq \min\{hod(A), hod(B)\}$.
- Pokud je lineární zobrazení izomorfizmus s maticí $A$ vzhledem k bázím $C, D$, potom inverzní zobrazení má vzhledem k bázím $D, C$ matici $A^{-1}$.